package simulation_Lab12Version;

import java.awt.Canvas;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.image.BufferStrategy;

/**
 * This class encapsulates a view. This implementation uses Java Swing
 * components (rather that RasterImage). As well, it uses double buffering, for
 * improved graphics performance.
 * 
 * @author mb
 * 
 */
public class View extends Canvas implements ModelListener {

	private Model theModel;

	/**
	 * Construct a view of the world, using the passed Model as the source of
	 * information about the world.
	 * 
	 * This view will install itself on the Model as a listener, to detect when
	 * the underlying model has changed.
	 */
	public View(Model theModel) {
		this.theModel = theModel;

		Frame theGameWindow = new Frame();
		theGameWindow.setUndecorated(false);
		theGameWindow.add(this);
		theGameWindow.setMinimumSize(theModel.getWorldDimension());
		theGameWindow.pack();
		theGameWindow.setVisible(true);
		this.createBufferStrategy(2);
		this.requestFocus();

		// here we install this view as a listener on the model.
		// this makes sense because this view depends upon the model in order to
		// know how to draw itself.
		theModel.addListener(this);

	}

	public void changed() {
		this.repaint();
	}

	/**
	 * This method accomplishes the painting of this canvas.
	 * 
	 * This method is called when the contents of the component should be
	 * painted; such as when the component is first being shown or is damaged
	 * and in need of repair.
	 * 
	 * The window manager, once alerted to the fact that a window needs to be
	 * redrawn, will trigger a process that will ultimately result in this
	 * method being invoked.
	 * 
	 * It should be understood that the main method of an app typically does not
	 * directly invoke this method. Rather, the application indicates that the
	 * canvas is in need of painting via the invalidate() or repaint() methods,
	 * which signals that the canvas needs to be redrawn.
	 * 
	 * The canvas will need to be redrawn, for instance, when the game state
	 * changes (like the user has moved a sprite or fired a shot or simply the
	 * next frame advances).
	 * 
	 */
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		drawFrame(g2);
	}

	/**
	 * This method implements the functionality so that the scene is drawn on
	 * the back buffer, and then the entire back buffer is shown at once.
	 * 
	 * This method accomplishes a performance improvement. Instead of the
	 * graphics being drawn repeatedly, for each time an on-screen element is
	 * specified (incrementally), the buffer is rendered all at once.
	 * 
	 * Implementation note: this method delegates to another method the
	 * specification of what the screen elements are.
	 * 
	 * @param g
	 *            the Graphics2D object that is encapsulating the graphics
	 *            context on which the on-screen elements are being drawn.
	 */
	public void drawFrame(Graphics2D g) {
		BufferStrategy bufStrategy = this.getBufferStrategy();
		Graphics2D backBuffer = (Graphics2D) bufStrategy.getDrawGraphics();

		// here we draw our game world (nicely delegated to another method)
		drawOnScreenElements(backBuffer);

		// Clean up
		backBuffer.dispose();
		bufStrategy.show();
		Toolkit.getDefaultToolkit().sync();
	}

	/**
	 * This method draws all of the required on-screen elements on the passed
	 * graphics context.
	 * 
	 * Implementation note: the client should not assume that the graphics
	 * context corresponds to what is shown on-screen. In fact, it may be a
	 * graphics buffer that will be rendered at some other point.
	 * 
	 * @param g
	 *            as specified above
	 */
	public void drawOnScreenElements(Graphics2D g) {
		theModel.drawSceneBackground(g);
		theModel.drawCurrentState(g);
	}

}
